// Copyright (c) 2002 Cunningham & Cunningham, Inc.
// Released under the terms of the GNU General Public License version 2 or later.

package fat;

import fit.*;

import java.net.URL;
import java.util.*;
import java.io.*;

public class Tests extends Fixture {

    Parse heads;
    String page;

    public void doRows(Parse rows) {
        heads = rows.parts;
        super.doRows(rows.more);
    }

    public void doCell(Parse cell, int column) {
        switch (column) {
            case(0):
                page = cell.text();
                break;
            default:
                String language = heads.at(column).text();
                String runscript = (String) Frameworks.runscripts.get(language);
                performTest(cell, runscript, page);
        }
    }

    public void performTest(Parse cell, String runscript, String page) {
        if (runscript == null || runscript.equals("null")  || page.startsWith("?")) {
            ignore(cell);
            return;
        }
        try {
            String pageUrl = "http://fit.c2.com/wiki.cgi?" + page;
            String testUrl = runscript + pageUrl;
            String testResult = get(new URL(testUrl));
            Counts counts = count(testResult);
            String message = anchor(counts.wrong+"/"+counts.exceptions+"&nbsp;", testUrl);
            cell.addToBody(message);
            if (counts.wrong==0 && counts.exceptions==0) {
                right(cell);
            } else {
                wrong(cell);
            }
        } catch (Throwable e) {
            exception(cell, e);
        }
    }

    protected String anchor(String body, String link) {
        return "<a href=\"" + link + "\">" + body + "</a>";
    }

    protected String get(URL url) throws IOException {
        InputStream stream =  (InputStream) url.getContent();
        BufferedReader reader = new BufferedReader(new InputStreamReader(stream));
        StringBuffer buffer = new StringBuffer(10000);
        for (String line; (line = reader.readLine()) != null; ) {
            buffer.append(line);
            buffer.append('\n');
        }
        return buffer.toString();
    }

    protected Counts count(String results) throws Exception {
        Counts counts = new Counts();
        Parse data = new Parse(results, new String[]{"td"});
        while (data != null) {
            Color c = Color.parse(data);
            data = data.more;
            if (c == null)          {continue;}
            else if (c.isGreen())   {counts.right++;}
            else if (c.isRed())     {counts.wrong++;}
            else if (c.isYellow())  {counts.exceptions++;}
            else if (c.isGray())    {counts.ignores++;}
        }
        return counts;
    }

    public static class Color {

        int r,g,b;

        public Color (int r, int g, int b) {
            this.r=r; this.g=g; this.b=b;
        }

        public static Color parse (Parse cell) {
            if (cell==null) {
                return null;
            } else {
                String pattern = " bgcolor=\"#";
                int index = cell.tag.indexOf(pattern);
                if (index >= 0) {
                    int len = pattern.length();
                    int rgb = Integer.parseInt(cell.tag.substring(index+len, index+len+6), 16);
                    return new Color(rgb>>16&255, rgb>>8&255, rgb&255);
                } else {
                    return null;
                }
            }
        }

        boolean isRed()     {return r > g && r > b;}
        boolean isGreen()   {return g > r && g > b;}
        boolean isYellow()  {return r > b && g > b;}
        boolean isGray()    {return r > b && g > b;}
    }

}
